#!/bin/sh

# gEDA - GPL Electronic Design Automation
# gEDA Binary Suite Installer
# Copyright (C) 2008 Ales Hvezda
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA

version=0.0.2
orig_installdir=/usr/local/geda-0.0.2

echo ""
echo "Welcome to the gEDA Binary Suite Installer for GNU/Linux x86"
echo "(version $version)"
echo ""
echo "At all the following prompts press ENTER to take the default."
echo ""
echo ""

echo "Enter install directory (default: $HOME/geda-$version)?"
echo "                         NOTE: If you want multiple users to use the"
echo "                               suite and you have a pre-dbus GNU/Linux"
echo "                               system, you must install the suite into:"
echo "                               $orig_installdir"
echo -n "> "
read installdir

if [ "$installdir" = "" ]
then
	installdir="$HOME/geda-$version"
fi

echo ""
echo "Install directory: $installdir"
echo ""

if [ "$installdir" = "/usr" ]
then
	echo "I will not let you install into $installdir.  Sorry, but"
	echo "doing so would probably break your OS.  Please pick a different"
 	echo "install directory like /usr/local/geda-$version."
	echo ""
	exit 1
fi

if [ -e "$installdir" ]
then
	echo "$installdir already exists.  Please remove it and rerun this script"
	echo ""
	exit 1
fi

if [ ! -d "$installdir" ]
then
	echo "$installdir does not exist."
	echo "Create $installdir (default: y)?"
	echo -n "> " 
	read createinstalldir

	if [ "$createinstalldir" = "" ]
	then
		createinstalldir=y
	fi

	if [ "$createinstalldir" != "y" ]
	then
		echo ""
		echo "You must allow me to create $installdir"
		echo "I am now exiting... Bye!"
		exit 1
	fi
fi

echo ""
echo "Are you sure you wish to install the gEDA Suite binaries (default: y)?"
echo -n "> "
read verify

if [ "$verify" = "" ]
then
	verify=y
fi

if [ "$verify" != "y" ]
then
	echo ""
	echo "Okay, I will not install the gEDA Suite binaries. Bye!"
	exit 1
fi

md5sum --help > /dev/null 2>&1
status=$?
if [ "$status" = 0 ]
then
	echo ""
	echo "Verifying files..."
	find dist -type f -exec md5sum {} \; > .local_md5sums
	md5sum install.sh README COPYING >> .local_md5sums
	cat .local_md5sums | sort > .sorted_local_md5sums
	cat .dist_md5sums | sort > .sorted_dist_md5sums
	diff .sorted_dist_md5sums .sorted_local_md5sums > /dev/null 2>&1
	status=$?
	if [ ! "$status" = 0 ]
	then
		echo ""
		echo "Integrity check failed.  Please verify your download. Exiting!"
		exit 1
	fi
else
	echo ""
	echo "md5sum not installed, cannot verify integrity of this distribution."
	echo "In order to verify the integrity of this distribution, please"
	echo "install md5sum."
	echo "Continue without this verification? (default: n)?"
	echo -n "> "
	read cont

	if [ "$cont" = "" ]
	then
		cont=n
	fi

	if [ "$cont" = "n" ]
	then
		echo ""
		echo "Okay, I will not install the gEDA Suite binaries. Bye!"
		exit 1
	fi
fi

echo "Creating: $installdir"
mkdir -p $installdir 
status=$?
if [ "$status" != 0 ]
then
	echo "Creation of $installdir failed!"
	echo "Exiting... Bye!"
	exit 1
fi

echo "Installing files..."
cp -rp dist/* $installdir
status=$?
if [ "$status" != 0 ]
then
	echo "Some files did not install correctly"
	echo "Exiting... Bye!"
	exit 1
fi

cd $installdir
echo "Performing post install setup..."
fixupfiles="./bin/execwrapper ./bin/pythonwrapper ./bin/gawkwrapper ./bin/perlwrapper ./etc/pango/pangorc ./etc/fonts/fonts.conf ./etc/pango/pango.modules ./etc/gtk-2.0/gdk-pixbuf.loaders ./etc/gtk-2.0/gtk.immodules ./etc/dbus-1/system.conf ./etc/rc.d/init.d/messagebus ./share/man/man1/gerbv.1 ./share/gEDA/scheme/gnet-PCBboard.scm ./share/gEDA/scheme/gnet-gsch2pcb.scm ./bin/gschemdoc ./bin/.scripts/garchive.py ./bin/.scripts/pygtk-demo.py ./bin/.scripts/xgsch2pcb.py"
for i in $fixupfiles
do
	mv $i $i.orig
	cat $i.orig | sed "s^%INSTALLDIR%^$installdir^g" > $i

	insidebin=`echo $i | grep bin`
	if [ "$insidebin" != "" ]
	then
		chmod 755 $i
	fi

	rm -f $i.orig
done

# figure out a manpath for the user to set
ls_manpage=`man -w ls`
system_manpath=`dirname $ls_manpage | sed "s^/man1^^g"`
possible_manpath=$installdir/share/man:$system_manpath

echo "Done!"
echo ""
echo "In order run the gEDA Binary Suite programs, change directory"
echo "into $installdir/bin and run the appropriate binaries \(such as geda,"
echo "gschem, or pcb\).  Also please read the documentation/tutorials at:"
echo "  $installdir/share/doc/geda-doc/wiki/index.html"
echo ""
echo "You may want to set your PATH variable to point to:"
echo "  $installdir/bin"
echo ""
echo "You do not need to set any other environment variables \(such as"
echo "LD_LIBRARY_PATH\).  Do not set your LD_LIBRARY_PATH to the directory"
echo "where you installed the gEDA Suite binaries.  Doing so might break"
echo "other non-gEDA programs."
echo ""
echo "You can set your PATH and MANPATH by executing one of the following"
echo "lines or by adding one of the following lines to the appropriate shell"
echo "initialization file:"
echo ""
echo "For vanilla bourne shells:"
echo "PATH=$installdir/bin:\$PATH"
echo "MANPATH=$possible_manpath:\$MANPATH"
echo "export PATH"
echo "export MANPATH"
echo ""
echo "For bash:"
echo "export PATH=$installdir/bin:\$PATH"
echo "export MANPATH=$possible_manpath:\$MANPATH"
echo ""
echo "For csh/tcsh:"
echo "setenv PATH $installdir/bin:\$PATH"
echo "setenv MANPATH $possible_manpath:\$MANPATH"
echo ""

